/*
 * Copyright (c) 1994 The University of Utah and
 * the Center for Software Science (CSS).  All rights reserved.
 *
 * Permission to use, copy, modify and distribute this software and its
 * documentation is hereby granted, provided that both the copyright
 * notice and this permission notice appear in all copies of the
 * software, derivative works or modified versions, and any portions
 * thereof, and that both notices appear in supporting documentation.
 *
 * THE UNIVERSITY OF UTAH AND CSS ALLOW FREE USE OF THIS SOFTWARE IN ITS "AS
 * IS" CONDITION.  THE UNIVERSITY OF UTAH AND CSS DISCLAIM ANY LIABILITY OF
 * ANY KIND FOR ANY DAMAGES WHATSOEVER RESULTING FROM THE USE OF THIS SOFTWARE.
 *
 * CSS requests users of this software to return to css-dist@cs.utah.edu any
 * improvements that they make and grant CSS redistribution rights.
 *
 *      Author: Bryan Ford, University of Utah CSS
 */
#ifndef _MACH_I386_IPC_H_
#define _MACH_I386_IPC_H_

/*
   This file defines the i386-specific aspects of Mach 4 IPC.

   On entry:

	eax		request code to send
	ebx	0-7	key to invoke
		8-15	slot for received key, 0 if none
		16-31	size of receive buffer, 0 if none
	ecx	0-7	key to send, 0 if none
		8-15	must be 0
		16-31	size of send buffer, 0 if none
	edx		pointer to send buffer
	esi		data value to send
	edi		data value to send
	ebp		pointer to receive buffer

   On return:

	eax		return code received
	ebx		undefined
	ecx	0-7	nonzero if a key was received
		8-15	undefined
		16-31	size of received message
	edx		value of start key's user data slot
			(undefined for resume keys)
	esi		data value received
	edi		data value received
	ebp		undefined
*/

/* Number of elements of each type that can be passed in one message.  */
#define IPC_MAX_VALUES		2
#define IPC_MAX_BLOCKS		1
#define IPC_MAX_KEYS		1

/* Maximum size of a data block.  */
/* (I'll probably increase this to 4K.)  */
#define IPC_MAX_BLOCK_SIZE	1024


/* Call gate selectors in the standard LDT for IPC on the i386.  */
#define	USER_IPC_CALL	0x27		/* CALL gate */
#define	USER_IPC_RETURN	0x2f		/* RETURN gate */
#define	USER_IPC_FORK	0x37		/* FORK gate */


#endif /* _MACH_I386_IPC_H_ */
